@@ -230,6 +230,7 @@ class ZiaTalker(object):
             url = '/urlCategories?customOnly=true'
         else:
             url = '/urlCategories'
+        #return self._obtain_all(url)
         response = self.hp_http.get_call(url, cookies={'JSESSIONID': self.jsessionid}, error_handling=True)
         return response.json()
 
@@ -242,18 +243,19 @@ class ZiaTalker(object):
         return response.json()
 
     def add_url_categories(self, name, superCategory, type='URL_CATEGORY', urls=None, dbCategorizedUrls=None,
-                           keywordsRetainingParentCategory=None, customCategory=False, ipRanges=None,
-                           ipRangesRetainingParentCategory=None):
+                           keywordsRetainingParentCategory=[], keywords=[], customCategory=False, ipRanges=[],
+                           ipRangesRetainingParentCategory=[]):
         """
          Adds a new custom URL category.
         :param name: type string. Name of the custom category. Possible values URL_CATEGORY, TLD_CATEGORY, ALL
         :param superCategory: type string. super category
         :param urls: type list. List of urls
         :param dbCategorizedUrls: type list. URL retaining parent category
-        :param keywordsRetainingParentCategory: type list. List of key works
+        :param keywordsRetainingParentCategory: type list. Retained custom keywords from the parent URL category that is associated to a URL category.
+        :param keywords: type list. Custom keywords associated to a URL category.
         :param customCategory: type boolean. Default False. Set to True for custom category
-        :param IpRanges: type list. Custom IP address ranges associated to a URL category
-        :param ipRangesRetainingParentCategory: The retaining parent custom IP address ranges associated to a URL category
+        :param ipRanges: type list. Custom IP address ranges associated to a URL category
+        :param ipRangesRetainingParentCategory: The retaining parent custom IP address ranges associated to a URL category.
 
         :return:  json
         """
@@ -271,6 +273,7 @@ class ZiaTalker(object):
             "customCategory": customCategory,
             "superCategory": superCategory,
             "keywordsRetainingParentCategory": keywordsRetainingParentCategory,
+            "keywords": keywords,
             "urls": urls,
             "dbCategorizedUrls": dbCategorizedUrls,
             "ipRanges": ipRanges,
@@ -1289,7 +1292,7 @@ class ZiaTalker(object):
 
     def add_firewallFilteringRules(self, name, order, state, action, description=None, defaultRule=False,
                                    predefined=False, srcIps=None, destAddresses=None, destIpGroups=None,
-                                   srcIpGroups=None, labels=None, rank=0):
+                                   srcIpGroups=None, destIpCategories=None, labels=None,nwServices=None, rank=0):
         """
         :param name: type str,  Name of the Firewall Filtering policy rule ["String"]
         :param order: type int, Rule order number of the Firewall Filtering policy rule
@@ -1302,7 +1305,9 @@ class ZiaTalker(object):
         :param srcIps: type list, List of source IP addresses
         :param destAddresses: type list. List of destination addresses
         :param destIpGroups: type list: List of user-definied destination IP address groups
-        :param srcIpGroups: type list: List of user defined source IP addres groups
+        :param srcIpGroups: type list: List of user defined source IP address groups
+        :param destIpCategories: type list: list of destination IP categories
+        :param nwServices: type list. List of user-defined network services on whih the rule is applied
         :return: Default is false.If set to true, a predefined rule is applied
         """
 
@@ -1330,6 +1335,10 @@ class ZiaTalker(object):
             payload.update(destIpGroups=destIpGroups)
         if labels:
             payload.update(labels=labels)
+        if destIpCategories:
+            payload.update(destIpCategories=destIpCategories)
+        if nwServices:
+            payload.update(nwServices=nwServices)
         response = self.hp_http.post_call(url, payload=payload, cookies={'JSESSIONID': self.jsessionid},
                                           error_handling=True)
         return response
@@ -1438,7 +1447,7 @@ class ZiaTalker(object):
     def add_ipDestinationGroups(self, name, type, addresses, ipCategories=None, countries=None, description=None):
         """
         :param name: mame
-        :param type: Destination IP group type. Either DSTN_IP or DSTN_FQDN
+        :param type: Destination IP group type. Either DSTN_IP or  DSTN_FQDN or DSTN_DOMAIN
         :param addresses: List of Destination IP addresses within the group.
         :param description: description
         :param ipCategories: List of Destination IP address URL categories. You can identify destination based
@@ -1446,7 +1455,7 @@ class ZiaTalker(object):
         :param countries: list of destination IP address countries. You can identify destinations based on the location
         of a server.Default value ANY
         """
-        if type not in ["DSTN_IP", "DSTN_FQDN"]:
+        if type not in ["DSTN_IP", "DSTN_FQDN", "DSTN_DOMAIN"]:
             raise ValueError("Invalid destination type ")
         if countries:
             for i in countries:
